/*  
	     1,  OC1B (   PB2)
	      LTK v2
	5  2015  ltagkirov
*/

#include "globals.h"

#include "config.h"
#include "miles.h"

uchar previous_ir_time;
struct IRPackage current_ir;
uchar ir_bits_recieved=0;
uchar ir_recieve_mask=0b10000000;



//  send_ir_xxx  2.        delay.h
#define WAIT_USING_TIMER1B



#ifdef WAIT_USING_TIMER1B

	#define _us1(X) (CARRIER_FREQ/1000+1)*2*X/1000
	#define _us0(X) X/31.25

	//       56,     uchar
	//   
	volatile uint delay_ticks_left=0;

	ISR(TIMER1_COMP_vect) {
		if (delay_ticks_left != 0) {
			delay_ticks_left--;
		};
	};


	#define wait_us(ticks)\
	do {\
		delay_ticks_left = ticks;\
		TIMSK |= (1<<OCIE1B);\
		while (delay_ticks_left != 0);\
		TIMSK &= ~(1<<OCIE1B);\
	} while (0)

#else

	// .   - .   ( 1)  .
	#include <util/delay.h>
	#define _us1(X) X
	#define _us0(X) X
	#define wait_us(X) _delay_us(X)

#endif



//  
//WGM21 = CTC
//COM20 = Toggle OC2 on Compare Match
//CS20	= prescaler clk/1

// COM1B = 10    OC1B
#define enable_fire_pwm(ticks)\
do {\
	TCNT1 = 0;\
	TCCR1A |= 0x20;\								
	wait_us(_us1(ticks));\
} while (0)



//   
//    
// COM1B = 00    OC1B
#define disable_fire_pwm(ticks)\
do {\
	TCCR1A &= 0xCF;\ 								
	TCNT1 = 0;\
	DISABLE_OUTPUT(B,2,NORMAL);\
	wait_us(_us0(ticks));\
} while(0)




void send_ir_start() {
	enable_fire_pwm		(2400);
	disable_fire_pwm	(600);
};

void send_ir_1() {
	enable_fire_pwm		(1200);
	disable_fire_pwm	(600);
};

void send_ir_0() {
	enable_fire_pwm		(600);
	disable_fire_pwm	(600);
};

void send_ir_bits (const uchar data, uchar mask) {
  while (mask) {
    if (data&mask) {
      send_ir_1();
    } else {
      send_ir_0();
    };
    mask>>=1;
  };
};

void send_ir_fire (const uchar id, const uchar team, const uchar damage) {
	send_ir_start();
	send_ir_0();
	send_ir_bits(id,		0b01000000);
	send_ir_bits(team,	0b00000010);
	send_ir_bits(damage,0b00001000);
};

void send_ir_command (const uchar byte1, const uchar byte2, const uchar byte3) {
	send_ir_start();
	send_ir_bits(byte1,	0b10000000);
	send_ir_bits(byte2,	0b10000000);
	send_ir_bits(byte3,	0b10000000);
};


//   IR.   ,
SIGNAL (SIG_INTERRUPT0) {
	uchar current_time = TCNT0;
	uchar current_bit = 0;

	if (CHECK_INPUT(D, 2, NORMAL)) {
		current_bit = 1;
	};

	uchar delta_time;
	delta_time = current_time - previous_ir_time;
	previous_ir_time = current_time;

	//   1  0 -     .          
	if (current_bit) {
		return;
	};

	//   
	//2400us -  75    31250
	// +-480us   
	if ((delta_time>60)&&(delta_time<90)) {

		//       3   ,
		if (ir_bits_recieved>3) {
			if (incoming_ir.state == PACKAGE_EMPTY) {
				incoming_ir.state = PACKAGE_BROKEN;
				incoming_ir.bytes[0] = current_ir.bytes[0];
				incoming_ir.bytes[1] = current_ir.bytes[1];
				incoming_ir.bytes[2] = current_ir.bytes[2];
			};
		};

		ir_bits_recieved = 0;
		ir_recieve_mask = 0b10000000;
		//    
		current_ir.bytes[0] = 0;
		current_ir.bytes[1] = 0;
		current_ir.bytes[2] = 0;

		return;
	};


	//  ,    -     
	if (delta_time>13 && delta_time<25) {
		// 0 - 448us..832us
		// ,    -        ,    
	} else if (delta_time>32 && delta_time<48) {
		// 1 = 1056us..1568us
		// ,    
		current_ir.bytes[ir_bits_recieved>>3] |= ir_recieve_mask;
	} else {
		//       ,   
		//     ,   -  
		//  ,     -     
		if (ir_bits_recieved>3) {
			if (incoming_ir.state == PACKAGE_EMPTY) {
				incoming_ir.bytes[0] = current_ir.bytes[0];
				incoming_ir.bytes[1] = current_ir.bytes[1];
				incoming_ir.bytes[2] = current_ir.bytes[2];
				incoming_ir.state = PACKAGE_BROKEN;
			};
		};
		ir_bits_recieved=0;
		return;
	};

	//    ,   
	ir_recieve_mask = ir_recieve_mask>>1;
	if (ir_recieve_mask==0) {
		ir_recieve_mask = 0b10000000;
	};
	ir_bits_recieved++;

	// 14    24  .     
	if (ir_bits_recieved == 24 || (ir_bits_recieved == 14 && ((current_ir.bytes[0] & 0b10000000)==0 ) )) {
		if (incoming_ir.state == PACKAGE_EMPTY) {
			//  ,  .   24    14  .
			//     
			incoming_ir.bytes[0] = current_ir.bytes[0];
			incoming_ir.bytes[1] = current_ir.bytes[1];
			incoming_ir.bytes[2] = current_ir.bytes[2];

			if (ir_bits_recieved == 14) {
				//    union  ,      
				incoming_ir.id			= (current_ir.bytes[0]&0b01111111);
				incoming_ir.team		= (current_ir.bytes[1]&0b11000000)>>6;
				incoming_ir.damage	= (current_ir.bytes[1]&0b00111100)>>2;
				incoming_ir.state 	= PACKAGE_SHOOT;
			} else {
				incoming_ir.state = PACKAGE_COMMAND;
			};
		} else {
			//  -      
			//   .    
			//  -    , ..    
			//    ,     
		};
		//    ,    
		ir_bits_recieved = 0;
	};


} //   
